package spiderweb;

public class Point {
	private double x;
	private double y;
	
	public Point(double x, double y) {
		this.x = x;
		this.y = y;
	}
	
	public double distance(Point ot) {
		return Math.sqrt((x - ot.x) * (x - ot.x) + (y - ot.y) * (y - ot.y));
	}
	
	public double getX() {
		return x;
	}

	public void setX(double x) {
		this.x = x;
	}

	public double getY() {
		return y;
	}

	public void setY(double y) {
		this.y = y;
	}

	public Point plus(Point ot) {
		return new Point(x + ot.x, y + ot.y);
	}
	
	public Point minus(Point ot) {
		return new Point(x - ot.x, y - ot.y);
	}
	
	public Point scalar(double d) {
		return new Point(x * d, y * d);
	}
	
	public double dot(Point ot) {
		return x * ot.x + y * ot.y;
	}
	
	public double det(Point ot) {
		return x * ot.y - y * ot.x;
	}
	
	public double angle(Point ot) {
		double ret = Math.acos(this.dot(ot) / (this.norm() * ot.norm()));
		ret = ret * 180.0 / Math.PI;
		return ret;
	}
	
	/* rotate point around center angle in counter-clockwise */
	public Point rotate(Point center, double _angle) {
		double angle = _angle * Math.PI / 180.0;
		double nx = center.x + Math.cos(angle) * (x - center.x) - Math.sin(angle) * (y - center.y);
		double ny = center.y + Math.sin(angle) * (x - center.x) + Math.cos(angle) * (y - center.y);
		return new Point(nx, ny);
	}
	
	// intersect of pq and O-point
	public Point intersect(Point p, Point q) {
		double t = p.det(q) / (this.det(q) - this.det(p));
		return new Point(t * x, t * y);
	}
	
	public Point shrink(double len) {
		double t = len / this.norm();
		if (t >= 1) return null;
		return new Point(t * x, t * y);
	}
	
	public double norm() {
		return Math.sqrt(x * x + y * y);
	}
	
	public String toString() {
		return "x = " + x + ", y = " + y;
	}
}