package spiderweb;

import java.util.ArrayList;
import java.util.Arrays;

public class Population {
	private final static int NRADIUS_MIN = 4;
	private final static int NRADIUS_MAX = 10;
	private final static int BIAS_ANGLE_MIN = 0;
	private final static int BIAS_ANGLE_MAX = 90;
	private final static int BIAS_ANGLE_MIN1 = -10;
	private final static int BIAS_ANGLE_MAX1 = 10;
	private final static int LEN_MIN = 40;
	private final static int LEN_MAX = 300;
	private final static int ANGLE_MIN = 5;
	private final static int ANGLE_MAX = 30;
	private final static int DSTART_MIN = 2;
	private final static int DSTART_MAX = 10;
	private final static int DLEN_MIN = 3;
	private final static int DLEN_MAX = 10;
	
	private double eliteRate;
	private double mutationRate;
	
	private Circle[] insects;
	
	private MersenneTwisterFast random;
	private int numberIndividuals;
	private int iterations;
	private WebGene[] individuals;
	public WebGene bestIndividual;
	public WebGene currentIndividual;
	private int count;
	
	private String[] svgOfEachGeneration;
	
	public Population(int numberIndividuals, int iterations, int numberInsect, double insectSize, double eliteRate, double mutationRate, long seed) {
		this.numberIndividuals = numberIndividuals;
		this.iterations = iterations;
		this.eliteRate = eliteRate;
		this.mutationRate = mutationRate;
		this.svgOfEachGeneration = new String[iterations];
		this.count = 0;
		random = new MersenneTwisterFast(seed);
		
		this.individuals = new WebGene[numberIndividuals];
		for (int i = 0; i < numberIndividuals; ++i) {
			individuals[i] = new WebGene();
		}
		bestIndividual = new WebGene();
		currentIndividual = new WebGene();
		
		this.insects = new Circle[numberInsect];
		for (int i = 0; i < numberInsect; ++i) {
			insects[i] = new Circle(new Point(this.rangeRandom(-LEN_MAX, LEN_MAX), this.rangeRandom(-LEN_MAX, LEN_MAX)), insectSize);
		}
	}
	
	public void initialize() {
		for (int i = 0; i < this.numberIndividuals; ++i) {
			// number of radiuses
			individuals[i].gene[0] = this.rangeRandom(NRADIUS_MIN, NRADIUS_MAX);
			
			// bias angle of first radius
			individuals[i].gene[1] = this.rangeRandom(BIAS_ANGLE_MIN, BIAS_ANGLE_MAX);
			
			// length of radius
			for (int j = 2; j < 12; ++j) {
				individuals[i].gene[j] = this.rangeRandom(LEN_MIN, LEN_MAX);
			}
			
			// angle
			for (int j = 12; j < 22; ++j) {
				individuals[i].gene[j] = this.rangeRandom(ANGLE_MIN, ANGLE_MAX);
			}
			
			// bias angle for each radius
			for (int j = 22; j < 32; ++j) {
				individuals[i].gene[j] = this.rangeRandom(BIAS_ANGLE_MIN1, BIAS_ANGLE_MAX1);
			}
			
			// d-start and d-end
			individuals[i].gene[32] = this.rangeRandom(DSTART_MIN, DSTART_MAX);
			individuals[i].gene[33] = individuals[i].gene[32] + this.rangeRandom(DLEN_MIN, DLEN_MAX);
		}
	}
	
	public void run() {
		for (int i = 0; i < this.iterations; ++i) {
			nextGeneration();
			this.svgOfEachGeneration[i] = this.bestIndividual.toSvg();
			System.out.println("[Generation] = " + i + ", [Fitness] = " + this.bestIndividual.fitness);
		}
	}
	
	public int getParent(double sum) {
		double value = sum * random.nextDouble();
		for (int i = 0; i < this.numberIndividuals; ++i) {
			value -= individuals[i].fitness;
			if (value <= 0) return i;
		}
		return this.numberIndividuals - 1;
	}
	
	public void nextGeneration() {
		
		double sum = 0;
		for (int i = 0; i < this.numberIndividuals; ++i) {
			sum += individuals[i].fitness(insects);
		}
		Arrays.sort(individuals);
				
		if (individuals[0].fitness > this.bestIndividual.fitness) {
			this.bestIndividual = individuals[0];
		}
		this.svgOfEachGeneration[this.count++] = this.bestIndividual.toSvg();
		this.currentIndividual = individuals[0];
		
		WebGene[] next = new WebGene[this.numberIndividuals];
		for (int i = 0; i < this.numberIndividuals; ++i) {
			next[i] = new WebGene();
		}
		
		int numElites = (int) (this.numberIndividuals * this.eliteRate);
		if ((this.numberIndividuals - numElites) % 2 == 1) numElites++;
		
		for (int i = 0; i < numElites; ++i) {
			next[i] = individuals[i];
		}
		
		for (int i = numElites; i < this.numberIndividuals; i += 2) {
			int parent1 = this.getParent(sum);
			int parent2 = this.getParent(sum);
			while (parent2 == parent1) {
				parent2 = this.getParent(sum);
			}
			WebGene child1 = new WebGene();
			WebGene child2 = new WebGene();
			this.crossOver(parent1, parent2, child1, child2);
			this.mutate(child1);
			this.mutate(child2);
			this.repair(child1);
			this.repair(child2);
			next[i] = child1;
			next[i + 1] = child2;
		}
		
		this.individuals = next;
	}
	
	public void mutate(WebGene x) {
		for (int i = 0; i < WebGene.GENE_LEN; ++i) {
			if (random.nextDouble() < this.mutationRate) {
				if (i == 0) {
					x.gene[i] = this.rangeRandom(NRADIUS_MIN, NRADIUS_MAX);
				}
				if (i == 1) {
					x.gene[1] = this.rangeRandom(BIAS_ANGLE_MIN, BIAS_ANGLE_MAX);
				}
				if (i >= 2 && i < 12) {
					x.gene[i] = this.rangeRandom(LEN_MIN, LEN_MAX);
				}
				if (i >= 12 && i < 22) {
					x.gene[i] = this.rangeRandom(ANGLE_MIN, ANGLE_MAX);
				}
				if (i >= 22 && i < 32) {
					x.gene[i] = this.rangeRandom(BIAS_ANGLE_MIN1, BIAS_ANGLE_MAX1);
				}
				if (i == 32) x.gene[32] = this.rangeRandom(DSTART_MIN, DSTART_MAX);
				if (i == 33) x.gene[33] = individuals[i].gene[32] + this.rangeRandom(DLEN_MIN, DLEN_MAX);
			}
		}
	}
	
	public void crossOver(final int parent1, final int parent2, WebGene child1, WebGene child2) {
		int ind = this.rangeRandom(1, WebGene.GENE_LEN - 1);
		for (int i = 0; i < ind; ++i) {
			child1.gene[i] = individuals[parent1].gene[i];
			child2.gene[i] = individuals[parent2].gene[i];
		}
		for (int i = ind; i < WebGene.GENE_LEN; ++i) {
			child1.gene[i] = individuals[parent2].gene[i];
			child2.gene[i] = individuals[parent1].gene[i];
		}
	}
	
	public void repair(WebGene x) {
		if (x.gene[32] >= x.gene[33]) {
			x.gene[32] = this.rangeRandom(DSTART_MIN, DSTART_MAX);
			x.gene[33] = x.gene[32] + this.rangeRandom(DLEN_MIN, DLEN_MAX);
		}
	}
	
	public int rangeRandom(int l, int r) {
		int x = random.nextInt();
		x %= (r - l + 1);
		if (x < 0) x += r - l + 1;
		return l + (x % (r - l + 1));
	}
	
	public Circle[] getInsects() {
		return this.insects;
	}
	
	public double getCurrentFitness() {
		return this.currentIndividual.fitness;
	}
	
	public double getBestFitness() {
		return this.bestIndividual.fitness;
	}
	
	public ArrayList<Line> getCurrentLines() {
		return this.currentIndividual.getLines();
	}
	
	public ArrayList<Line> getBestLines() {
		return this.bestIndividual.getLines();
	}
	
	public String getSvg(int kthIterations) {
		return this.svgOfEachGeneration[kthIterations];
	}
}
