package spiderweb;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.ArrayList;

import javax.swing.*;

/**
 * 
 * @author Vu Van Tan
 */
public class Visualization extends JPanel implements Runnable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1338268977440424406L;
	private Population pop;
	private int numberIndividuals;
	private int iterations;
	private int numberInsect;
	private int delay;
	private long seed;
	
	private double insectSize;
	private double eliteRate;
	private double mutationRate;
	private Thread runner;
	private String location;
	private boolean stopped;
	

	/**
	 * Creates new form Visualization
	 */
	
	public Visualization() {
		initComponents();
		this.runner = new Thread(this);
	}
	
	private void init() {
		this.numberIndividuals = Integer.parseInt(this.txtPopulation.getText());
		assert(this.numberIndividuals > 0);
		this.iterations = Integer.parseInt(this.txtGeneration.getText());
		assert(this.iterations > 0);
		this.numberInsect = Integer.parseInt(this.txtNumInsect.getText());
		assert(this.numberInsect > 0);
		this.insectSize = Double.parseDouble(this.txtInsectSize.getText());
		assert(this.insectSize > 0);
		this.eliteRate = Double.parseDouble(this.txtElite.getText());
		assert(this.eliteRate >= 0);
		this.mutationRate = Double.parseDouble(this.txtMutation.getText());
		assert(this.mutationRate > 0);
		this.location = this.txtSaveTo.getText();
		this.stopped = false;
		this.delay = 100;	// ms
		this.seed = (long) (Long.MAX_VALUE * Math.random());
		
		this.pop = new Population(this.numberIndividuals, this.iterations, this.numberInsect, this.insectSize, this.eliteRate, this.mutationRate, this.seed);
	}
	
	public void run() {
		// TODO Auto-generated method stub
		
		this.cmdRun.setEnabled(false);
		this.cmdStop.setEnabled(true);
		this.cmdReset.setEnabled(false);
		init();
		int iters = 1;
		this.pop.initialize();
		
		while (iters <= this.iterations) {
			pop.nextGeneration();
			if (iters % 20 == 0) {
				PrintWriter writer;
				try {
					String fileName = location + "spider-" + iters + ".svg";
					writer = new PrintWriter(fileName, "UTF-8");
					writer.println(pop.getSvg(iters - 1));
					writer.close();
				} catch (FileNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (UnsupportedEncodingException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			drawWeb();
			this.txtCurrentFitness.setText(Double.toString(pop.getCurrentFitness()).substring(0, 10));
			this.txtOverAllFitness.setText(Double.toString(pop.getBestFitness()).substring(0, 10));
			this.txtGeno.setText(Integer.toString(iters));
			
			iters++;
			try {
				Thread.sleep(delay);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	private void drawWeb() {
		Graphics gCurr = (Graphics2D) this.currentPanel.getGraphics();
		int width = this.currentPanel.getWidth();
		int height = this.currentPanel.getHeight();
		gCurr.clearRect(0, 0, width, height);
		int biasX = width / 2;
		int biasY = height / 2;
		
		ArrayList<Line> lines = pop.getCurrentLines();
		for (int i = 0; i < lines.size(); ++i) {
			Line ln = lines.get(i);
			gCurr.drawLine(biasX + (int) ln.getStart().getX(), biasY + (int) ln.getStart().getY(), 
					biasX + (int) ln.getEnd().getX(), biasY + (int) ln.getEnd().getY());
		}
		
		Graphics gBest = (Graphics2D) this.bestPanel.getGraphics();
		width = this.bestPanel.getWidth();
		height = this.bestPanel.getHeight();
		gBest.clearRect(0, 0, width, height);
		biasX = width / 2;
		biasY = height / 2;
		
		lines = pop.getBestLines();
		for (int i = 0; i < lines.size(); ++i) {
			Line ln = lines.get(i);
			gBest.drawLine(biasX + (int) ln.getStart().getX(), biasY + (int) ln.getStart().getY(), 
					biasX + (int) ln.getEnd().getX(), biasY + (int) ln.getEnd().getY());
		}
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */

	private void initComponents() {
		inputPanel = new JPanel();
		lbPopulation = new JLabel();
		lbGeneration = new JLabel();
		lbElite = new JLabel();
		lbMutation = new JLabel();
		lbNumInsect = new JLabel();
		lbInsectSize = new JLabel();
		txtPopulation = new JTextField();
		txtGeneration = new JTextField();
		txtElite = new JTextField();
		txtMutation = new JTextField();
		txtNumInsect = new JTextField();
		txtInsectSize = new JTextField();
		lbParameters = new JLabel();
		commandPanel = new JPanel();
		cmdRun = new JButton();
		cmdStop = new JButton();
		cmdReset = new JButton();
		lbOperations = new JLabel();
		infoPanel = new JPanel();
		lbGeno = new JLabel();
		lbSaveTo = new JLabel();
		txtGeno = new JTextField();
		txtSaveTo = new JTextField();
		lbCurrentFitness = new JLabel();
		lbOverAllFitness = new JLabel();
		txtCurrentFitness = new JTextField();
		txtOverAllFitness = new JTextField();
		currentPanel = new JPanel();
		bestPanel = new JPanel();
		lbCurrentBest = new JLabel();
		lbOverAllBest = new JLabel();

		inputPanel.setBackground(new Color(153, 153, 153));

		lbPopulation.setText("Population");

		lbGeneration.setText("Generation");

		lbElite.setText("Elite");

		lbMutation.setText("Mutation");

		lbNumInsect.setText("Number Insect");

		lbInsectSize.setText("Insect's Size");

		txtPopulation.setText("1000");
		txtPopulation.setColumns(7);

		txtGeneration.setText("200");
		txtGeneration.setColumns(7);
		
		txtElite.setText("0.1");
		txtElite.setColumns(7);
		
		txtMutation.setText("0.1");
		txtMutation.setColumns(7);

		txtNumInsect.setText("400");
		txtNumInsect.setColumns(7);

		txtInsectSize.setText("1.0");
		txtInsectSize.setColumns(7);
		
		lbParameters.setFont(new Font("Lucida Grande", 0, 14));
		lbParameters.setText("Parameters");

		GroupLayout inputPanelLayout = new GroupLayout(inputPanel);
		inputPanel.setLayout(inputPanelLayout);
		inputPanelLayout
				.setHorizontalGroup(inputPanelLayout
						.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addGroup(
								inputPanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												inputPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.LEADING,
																false)
														.addComponent(
																lbInsectSize,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																lbNumInsect,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																lbMutation,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																lbElite,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																lbGeneration,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE)
														.addComponent(
																lbPopulation,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.DEFAULT_SIZE,
																Short.MAX_VALUE))
										.addPreferredGap(
												LayoutStyle.ComponentPlacement.RELATED,
												GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)
										.addGroup(
												inputPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.LEADING)
														.addComponent(
																txtPopulation,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																txtGeneration,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																txtElite,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																txtMutation,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																txtNumInsect,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE)
														.addComponent(
																txtInsectSize,
																GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE,
																GroupLayout.PREFERRED_SIZE))
										.addGap(27, 27, 27))
						.addGroup(
								inputPanelLayout
										.createSequentialGroup()
										.addGap(83, 83, 83)
										.addComponent(lbParameters)
										.addContainerGap(
												GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));
		inputPanelLayout.setVerticalGroup(inputPanelLayout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addGroup(
				inputPanelLayout
						.createSequentialGroup()
						.addComponent(lbParameters)
						.addGap(10, 10, 10)
						.addGroup(
								inputPanelLayout
										.createParallelGroup(
												GroupLayout.Alignment.BASELINE)
										.addComponent(lbPopulation)
										.addComponent(txtPopulation,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE))
						.addPreferredGap(
								LayoutStyle.ComponentPlacement.UNRELATED)
						.addGroup(
								inputPanelLayout
										.createParallelGroup(
												GroupLayout.Alignment.BASELINE)
										.addComponent(lbGeneration)
										.addComponent(txtGeneration,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE))
						.addPreferredGap(
								LayoutStyle.ComponentPlacement.UNRELATED)
						.addGroup(
								inputPanelLayout
										.createParallelGroup(
												GroupLayout.Alignment.BASELINE)
										.addComponent(lbElite)
										.addComponent(txtElite,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE))
						.addPreferredGap(
								LayoutStyle.ComponentPlacement.UNRELATED)
						.addGroup(
								inputPanelLayout
										.createParallelGroup(
												GroupLayout.Alignment.BASELINE)
										.addComponent(lbMutation)
										.addComponent(txtMutation,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE))
						.addPreferredGap(
								LayoutStyle.ComponentPlacement.UNRELATED)
						.addGroup(
								inputPanelLayout
										.createParallelGroup(
												GroupLayout.Alignment.BASELINE)
										.addComponent(lbNumInsect)
										.addComponent(txtNumInsect,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE))
						.addPreferredGap(
								LayoutStyle.ComponentPlacement.UNRELATED)
						.addGroup(
								inputPanelLayout
										.createParallelGroup(
												GroupLayout.Alignment.BASELINE)
										.addComponent(lbInsectSize)
										.addComponent(txtInsectSize,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE))
						.addContainerGap(GroupLayout.DEFAULT_SIZE,
								Short.MAX_VALUE)));

		commandPanel.setBackground(new Color(153, 153, 153));

		cmdRun.setText("Run");
		cmdRun.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				runActionPerformed(evt);
			}
		});

		cmdStop.setText("Stop");
		cmdStop.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				stopActionPerformed(evt);
			}
		});

		cmdReset.setText("Reset");
		cmdReset.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				resetActionPerformed(evt);
			}
		});

		lbOperations.setFont(new Font("Lucida Grande", 0, 14)); // NOI18N
		lbOperations.setText("Operations");

		GroupLayout commandPanelLayout = new GroupLayout(commandPanel);
		commandPanel.setLayout(commandPanelLayout);
		commandPanelLayout
				.setHorizontalGroup(commandPanelLayout
						.createParallelGroup(GroupLayout.Alignment.CENTER)
						.addGroup(
								commandPanelLayout
										.createSequentialGroup()
										.addGroup(
												commandPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.CENTER)
														.addGroup(
																commandPanelLayout
																		.createSequentialGroup()
																		.addGap(30,
																				30,
																				30)
																		.addComponent(
																				cmdRun)
																		.addGap(30,
																				30,
																				30)
																		.addComponent(
																				cmdStop))
														.addGroup(
																commandPanelLayout
																		.createSequentialGroup()
																		.addGap(30,
																				30,
																				30)
																		.addComponent(
																				cmdReset))
														.addGroup(
																commandPanelLayout
																		.createSequentialGroup()
																		.addGap(30,
																				30,
																				30)
																		.addComponent(
																				lbOperations)))
										.addContainerGap(26, Short.MAX_VALUE)));
		commandPanelLayout
				.setVerticalGroup(commandPanelLayout
						.createParallelGroup(GroupLayout.Alignment.CENTER)
						.addGroup(
								commandPanelLayout
										.createSequentialGroup()
										.addComponent(lbOperations)
										.addGap(11, 11, 11)
										.addGroup(
												commandPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.BASELINE)
														.addComponent(cmdRun)
														.addComponent(cmdStop))
										.addPreferredGap(
												LayoutStyle.ComponentPlacement.RELATED)
										.addComponent(cmdReset)));

		infoPanel.setBackground(new Color(153, 153, 153));

		lbGeno.setText("No. Generation #");
		lbSaveTo.setText("Save To");

		txtGeno.setText("0");
		txtGeno.setColumns(4);
		txtGeno.setEditable(false);

		txtSaveTo.setText("");

		lbCurrentFitness.setText("Current Best Fitness");

		lbOverAllFitness.setText("Overall Best Fitness");

		txtCurrentFitness.setText("0.00000000");
		txtCurrentFitness.setColumns(6);
		txtCurrentFitness.setEditable(false);

		txtOverAllFitness.setText("0.00000000");
		txtOverAllFitness.setColumns(6);
		txtOverAllFitness.setEditable(false);

		GroupLayout infoPanelLayout = new GroupLayout(infoPanel);
		infoPanel.setLayout(infoPanelLayout);
		infoPanelLayout
				.setHorizontalGroup(infoPanelLayout
						.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addGroup(
								infoPanelLayout
										.createSequentialGroup()
										.addContainerGap()
										.addGroup(
												infoPanelLayout
														.createParallelGroup(
																GroupLayout.Alignment.LEADING,
																false)
														.addGroup(
																infoPanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				lbSaveTo)
																		.addGap(76,
																				76,
																				76)
																		.addComponent(
																				txtSaveTo))
														.addGroup(
																infoPanelLayout
																		.createSequentialGroup()
																		.addComponent(
																				lbGeno)
																		.addGap(18,
																				18,
																				18)
																		.addComponent(
																				txtGeno,
																				GroupLayout.PREFERRED_SIZE,
																				GroupLayout.DEFAULT_SIZE,
																				GroupLayout.PREFERRED_SIZE)
																		.addGap(46,
																				46,
																				46)
																		.addComponent(
																				lbCurrentFitness)
																		.addGap(27,
																				27,
																				27)
																		.addComponent(
																				txtCurrentFitness,
																				GroupLayout.PREFERRED_SIZE,
																				135,
																				GroupLayout.PREFERRED_SIZE)
																		.addGap(27,
																				27,
																				27)
																		.addComponent(
																				lbOverAllFitness)
																		.addGap(27,
																				27,
																				27)
																		.addComponent(
																				txtOverAllFitness,
																				GroupLayout.PREFERRED_SIZE,
																				135,
																				GroupLayout.PREFERRED_SIZE)))
										.addContainerGap(
												GroupLayout.DEFAULT_SIZE,
												Short.MAX_VALUE)));
		infoPanelLayout.setVerticalGroup(infoPanelLayout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addGroup(
				infoPanelLayout
						.createSequentialGroup()
						.addContainerGap()
						.addGroup(
								infoPanelLayout
										.createParallelGroup(
												GroupLayout.Alignment.BASELINE)
										.addComponent(lbGeno)
										.addComponent(txtGeno,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE)
										.addComponent(lbCurrentFitness)
										.addComponent(lbOverAllFitness)
										.addComponent(txtCurrentFitness,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE)
										.addComponent(txtOverAllFitness,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE))
						.addPreferredGap(
								LayoutStyle.ComponentPlacement.UNRELATED)
						.addGroup(
								infoPanelLayout
										.createParallelGroup(
												GroupLayout.Alignment.BASELINE)
										.addComponent(lbSaveTo)
										.addComponent(txtSaveTo,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE))
						.addContainerGap(GroupLayout.DEFAULT_SIZE,
								Short.MAX_VALUE)));

		GroupLayout currentPanelLayout = new GroupLayout(currentPanel);
		currentPanel.setLayout(currentPanelLayout);
		currentPanelLayout.setHorizontalGroup(currentPanelLayout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addGap(0, 305, Short.MAX_VALUE));
		currentPanelLayout.setVerticalGroup(currentPanelLayout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addGap(0, 305, Short.MAX_VALUE));

		GroupLayout bestPanelLayout = new GroupLayout(bestPanel);
		bestPanel.setLayout(bestPanelLayout);
		bestPanelLayout.setHorizontalGroup(bestPanelLayout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addGap(0, 305, Short.MAX_VALUE));
		bestPanelLayout.setVerticalGroup(bestPanelLayout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addGap(0, 0, Short.MAX_VALUE));

		lbCurrentBest.setText("Current Best");

		lbOverAllBest.setText("Overall Best");

		GroupLayout layout = new GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout
				.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap()
								.addGroup(
										layout.createParallelGroup(
												GroupLayout.Alignment.TRAILING,
												false)
												.addComponent(
														infoPanel,
														GroupLayout.DEFAULT_SIZE,
														GroupLayout.DEFAULT_SIZE,
														Short.MAX_VALUE)
												.addGroup(
														layout.createSequentialGroup()
																.addGroup(
																		layout.createParallelGroup(
																				GroupLayout.Alignment.LEADING,
																				false)
																				.addComponent(
																						inputPanel,
																						GroupLayout.DEFAULT_SIZE,
																						GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE)
																				.addComponent(
																						commandPanel,
																						GroupLayout.DEFAULT_SIZE,
																						GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE))
																.addGroup(
																		layout.createParallelGroup(
																				GroupLayout.Alignment.LEADING)
																				.addGroup(
																						layout.createSequentialGroup()
																								.addPreferredGap(
																										LayoutStyle.ComponentPlacement.RELATED)
																								.addComponent(
																										currentPanel,
																										GroupLayout.PREFERRED_SIZE,
																										GroupLayout.DEFAULT_SIZE,
																										GroupLayout.PREFERRED_SIZE)
																								.addPreferredGap(
																										LayoutStyle.ComponentPlacement.RELATED)
																								.addComponent(
																										bestPanel,
																										GroupLayout.PREFERRED_SIZE,
																										GroupLayout.DEFAULT_SIZE,
																										GroupLayout.PREFERRED_SIZE))
																				.addGroup(
																						layout.createSequentialGroup()
																								.addGap(121,
																										121,
																										121)
																								.addComponent(
																										lbCurrentBest)
																								.addGap(236,
																										236,
																										236)
																								.addComponent(
																										lbOverAllBest)))))
								.addContainerGap(GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE)));
		layout.setVerticalGroup(layout
				.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(
						layout.createSequentialGroup()
								.addContainerGap(GroupLayout.DEFAULT_SIZE,
										Short.MAX_VALUE)
								.addGroup(
										layout.createParallelGroup(
												GroupLayout.Alignment.LEADING)
												.addGroup(
														layout.createSequentialGroup()
																.addComponent(
																		inputPanel,
																		GroupLayout.PREFERRED_SIZE,
																		GroupLayout.DEFAULT_SIZE,
																		GroupLayout.PREFERRED_SIZE)
																.addPreferredGap(
																		LayoutStyle.ComponentPlacement.RELATED)
																.addComponent(
																		commandPanel,
																		GroupLayout.PREFERRED_SIZE,
																		GroupLayout.DEFAULT_SIZE,
																		GroupLayout.PREFERRED_SIZE))
												.addGroup(
														layout.createSequentialGroup()
																.addGroup(
																		layout.createParallelGroup(
																				GroupLayout.Alignment.LEADING,
																				false)
																				.addComponent(
																						currentPanel,
																						GroupLayout.DEFAULT_SIZE,
																						GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE)
																				.addComponent(
																						bestPanel,
																						GroupLayout.DEFAULT_SIZE,
																						GroupLayout.DEFAULT_SIZE,
																						Short.MAX_VALUE))
																.addGap(18, 18,
																		18)
																.addGroup(
																		layout.createParallelGroup(
																				GroupLayout.Alignment.BASELINE)
																				.addComponent(
																						lbCurrentBest)
																				.addComponent(
																						lbOverAllBest))))
								.addPreferredGap(
										LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(infoPanel,
										GroupLayout.PREFERRED_SIZE,
										GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE)));
	}

	protected void resetActionPerformed(ActionEvent evt) {
		// TODO Auto-generated method stub
		this.runner = new Thread(this);
		this.stopped = false;

		this.txtElite.setEditable(true);
		this.txtGeneration.setEditable(true);
		this.txtPopulation.setEditable(true);
		this.txtInsectSize.setEditable(true);
		this.txtMutation.setEditable(true);
		this.txtNumInsect.setEditable(true);
		this.txtSaveTo.setEditable(true);
		this.txtGeno.setText("0");
		this.txtCurrentFitness.setText("0.00000000");
		this.txtOverAllFitness.setText("0.00000000");
	}

	@SuppressWarnings("deprecation")
	protected void stopActionPerformed(ActionEvent evt) {
		// TODO Auto-generated method stub
		if (this.stopped) return;
		this.runner.suspend();
		this.stopped = true;
		this.cmdRun.setEnabled(true);
		this.cmdReset.setEnabled(true);
	}

	@SuppressWarnings("deprecation")
	protected void runActionPerformed(ActionEvent evt) {
		// TODO add your handling code here:
		this.cmdRun.setEnabled(false);
		this.cmdStop.setEnabled(true);
		this.cmdReset.setEnabled(false);
		
		this.txtCurrentFitness.setEditable(false);
		this.txtElite.setEditable(false);
		this.txtGeneration.setEditable(false);
		this.txtPopulation.setEditable(false);
		this.txtInsectSize.setEditable(false);
		this.txtMutation.setEditable(false);
		this.txtNumInsect.setEditable(false);
		this.txtSaveTo.setEditable(false);
		
		if (this.stopped) {
			this.stopped = false;
			this.runner.resume();
		}
		else this.runner.start();
	}

	// Variables declaration - do not modify
	private JButton cmdRun;
	private JButton cmdStop;
	private JButton cmdReset;
	private JLabel lbPopulation;
	private JLabel lbSaveTo;
	private JLabel lbCurrentFitness;
	private JLabel lbOverAllFitness;
	private JLabel lbCurrentBest;
	private JLabel lbOverAllBest;
	private JLabel lbGeneration;
	private JLabel lbElite;
	private JLabel lbMutation;
	private JLabel lbNumInsect;
	private JLabel lbInsectSize;
	private JLabel lbParameters;
	private JLabel lbOperations;
	private JLabel lbGeno;
	private JPanel inputPanel;
	private JPanel commandPanel;
	private JPanel infoPanel;
	private JPanel currentPanel;
	private JPanel bestPanel;
	private JTextField txtPopulation;
	private JTextField txtOverAllFitness;
	private JTextField txtGeneration;
	private JTextField txtElite;
	private JTextField txtMutation;
	private JTextField txtNumInsect;
	private JTextField txtInsectSize;
	private JTextField txtGeno;
	private JTextField txtSaveTo;
	private JTextField txtCurrentFitness;

	// End of variables declaration

	public static void main(String[] args) {
		Visualization vis = new Visualization();
		JFrame frame = new JFrame();
		frame.setLayout(new BorderLayout());
		frame.add(vis, BorderLayout.CENTER);
		frame.pack();
		frame.setVisible(true);
		frame.setResizable(false);
		frame.setLocation(200, 0);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
}